/**
 * 1> 发起更新请求， 获得更新结果
 *  请求参数: {
					bootloader: bootloader,

					// apk | ipa: todo: 目前都是按android定义，通用怎么办？
					applicationId: applicationId,  // 应用id
					versionName: versionName,
					versionCode: versionCode,

					// 5+ manifest.json
					version: plus.runtime.version,  // version.name
					appid: plus.runtime.appid,		// id
				}
 * 2> 获取失败：
 *      失败处理
 * 3> 获取成功，处理更新结果：
 *                    updateType
 *      . 不需要更新      none            正常开始游戏
 *      . 整包更新       apk              http://ask.dcloud.net.cn/article/431
 *      . 资源包更新      wgt             http://ask.dcloud.net.cn/article/182
 *      . 资源差量更新    wgtu             http://ask.dcloud.net.cn/article/199
 *      . 网络进入        h5              更改首页地址并刷新
 *
 *
 * 4. 不同更新方案的技术差异：
 *      apk： 整包更新，更新后app会重新启动，所以不用考虑其他任何假设
 *      wgt, wgtu: 都可能更新 manifest.json, 其指向的首页（bootloader）等；但是都（可能）不需要重启app，所以更新后要重新加载
 *      h5: 本次过程不更改任何本地资源，使得重启后逻辑恢复
 *      除了 none, h5方式， 其他更新方式完成后都要确保5+ runtime会重新从解析manifest.json开始,因为更新逻辑本身也可能被更新
 * 5. TODO： 考虑在service worker中进行更新操作
 */
window.heleAppUpdate = (function () {
    /** 从服务器获取更新数据
     *  @serverUrl:更新文件地址
     *  @info: 本地版本信息
     *  @startGame: 开始游戏函数
     */
    function checkUpdate(serverUrl, info, startGame) {
        showText("检测更新");
        var args = [];
        for (var k in info) {
            var v = info[k];
            args.push(k + '=' + v);
        }
        serverUrl += '?' + args.join('&');
        getUpdateCommand(serverUrl, resultCB);
        /*
            {
                "updateType": "wgtu",
                "url": "http://blablabla.com/a.wgtu"
            }
        */
        function resultCB(result) {
            switch (result.updateType) {
                case 'none':
                    startGame();
                    break;
                case 'apk':
                case 'wgt':
                case 'wgtu':
                    downloadAndUpdate(result.url, result.updateType);
                    break;
                case 'h5':
                    window.location.replace(result.url);
                    break;
                default:
                    alert("无法处理的更新方式：" + result.updateType);
            }
        }
    }

    /* 1. 发起更新请求， 获得更新结果 */
    function getUpdateCommand(serverUrl, resultCB) {
        var xhr = new XMLHttpRequest();
        xhr.onreadystatechange = function () {
            switch (xhr.readyState) {
                case 4:
                    if (xhr.status == 200) {
                        var updateInfo = JSON.parse(xhr.responseText);
                        resultCB(updateInfo);
                    } else {
                        resultCB({ updateType: 'none' }); // 若无法检测更新，直接进入
                        // processError("检测更新失败,是否重试？");
                    }
                    break;
            }
        }
        xhr.open('GET', serverUrl);
        xhr.send();
    }

    // 下载文件
    function downloadAndUpdate(url, updateType) {
        showText("下载更新资源");
        var dtask = plus.downloader.createDownload(url, { filename: "_doc/update/" }, function (d, status) {
            if (status == 200) {
                installUpdate(d.filename);
            } else {
                processError("资源下载失败，是否重试？");
            }
        });
        //可通过以下方法获取下载的进度：
        dtask.addEventListener("statechanged", function () {
            var downSize = dtask.downloadedSize; // 已下载大小
            var totalSize = dtask.totalSize; // 总大小
            if (downSize != totalSize) {
                let pe = Math.ceil(downSize / totalSize * 100) + "%(" + Math.ceil(totalSize / 1000 / 10) / 100 + "MB)";
                showText("下载更新资源：" + pe);
            } else {
                showText("正在安装，请稍候");
            }
        }, false);
        dtask.start();
    }

    // 3种更新: apk, wgt, wgtu
    // 不检查版本, 所以支持降级
    function installUpdate(path) {
        plus.runtime.install(path, { force: true },
            function success(info) {
                // alert("应用资源更新完成！")
                plus.runtime.restart();
            },
            function failed(e) {
                alert("更新失败[" + e.code + "]：" + e.message);
                exitApp();
            }
        );
    }

    function processError(message) {
        if (confirm(message)) {
            location.reload();
        } else {
            exitApp();
        }
    }

    function exitApp() {
        let main = plus.android.runtimeMainActivity();
        main.finish();
    }

    function showText(string) {
        var text = document.getElementById("loadText");
        text.textContent = string;
    }

    return { checkUpdate: checkUpdate };
})();
